
// $Id: cli.c 884 2007-06-27 14:56:21Z sbalea $

/*
 * app_conference
 *
 * A channel independent conference application for Asterisk
 *
 * Copyright (C) 2002, 2003 Junghanns.NET GmbH
 * Copyright (C) 2003, 2004 HorizonLive.com, Inc.
 * Copyright (C) 2005, 2006 HorizonWimba, Inc.
 * Copyright (C) 2007 Wimba, Inc.
 *
 * Klaus-Peter Junghanns <kapejod@ns1.jnetdns.de>
 *
 * Video Conferencing support added by
 * Neil Stratford <neils@vipadia.com>
 * Copyright (C) 2005, 2005 Vipadia Limited
 *
 * VAD driven video conferencing, text message support
 * and miscellaneous enhancements added by
 * Mihai Balea <mihai at hates dot ms>
 *
 * This program may be modified and distributed under the
 * terms of the GNU General Public License. You should have received
 * a copy of the GNU General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision$")

#include "asterisk/autoconfig.h"
#include "cli.h"

static char *handle_cli_app_conference_restart(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference restart";
		e->usage = "usage: conference restart\n"
			" restart a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 2 )
		return CLI_SHOWUSAGE;
	
	kick_all();
	
	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_debug(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{

	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference debug";
		e->usage = "usage: conference debug <conference_name> [ on | off ]\n"
			" enable debugging for a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 3)
		return CLI_SHOWUSAGE;

	// get the conference name
	const char* name = a->argv[2] ;

   	// get the new state
	int state = 0 ;

	if ( a->argc == 3 )
	{
		// no state specified, so toggle it
		state = -1 ;
	}
	else
	{
		if ( strncasecmp( a->argv[3], "on", 4 ) == 0 )
			state = 1 ;
		else if ( strncasecmp( a->argv[3], "off", 3 ) == 0 )
			state = 0 ;
		else
			return CLI_SHOWUSAGE ;
	}

	int new_state = set_conference_debugging( name, state ) ;

	if ( new_state == 1 )
	{
		ast_cli( a->fd, "enabled conference debugging, name => %s, new_state => %d\n",
			 name, new_state ) ;
	}
	else if ( new_state == 0 )
	{
		ast_cli( a->fd, "disabled conference debugging, name => %s, new_state => %d\n",
			name, new_state ) ;
	}
	else
	{
		// error setting state
		ast_cli( a->fd, "\nunable to set debugging state, name => %s\n\n", name ) ;
	}

	return CLI_SUCCESS ;
}

static int conference_show_stats_name( int fd, const char* name )
{
	// not implemented yet
	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_show_stats(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{

	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference show stats";
		e->usage = "usage: conference show stats\n"
			" display stats for active conferences.\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 3 )
		return CLI_SHOWUSAGE ;

	// get count of active conferences
	int count = get_conference_count() ;

	ast_cli( a->fd, "\n\nCONFERENCE STATS, ACTIVE( %d )\n\n", count ) ;

	// if zero, go no further
	if ( count <= 0 )
		return CLI_SUCCESS ;

	//
	// get the conference stats
	//

	// array of stats structs
	ast_conference_stats stats[ count ] ;

	// get stats structs
	count = get_conference_stats( stats, count ) ;

	// make sure we were able to fetch some
	if ( count <= 0 )
	{
		ast_cli( a->fd, "!!! error fetching conference stats, available => %d !!!\n", count ) ;
		return CLI_SUCCESS ;
	}

	//
	// output the conference stats
	//

	// output header
	ast_cli( a->fd, "%-20.20s  %-40.40s\n", "Name", "Stats") ;
	ast_cli( a->fd, "%-20.20s  %-40.40s\n", "----", "-----") ;

	ast_conference_stats* s = NULL ;

	int i;

	for ( i = 0 ; i < count ; ++i )
	{
		s = &(stats[i]) ;

		// output this conferences stats
		ast_cli( a->fd, "%-20.20s\n", (char*)( &(s->name) )) ;
	}

	ast_cli( a->fd, "\n" ) ;

	//
	// drill down to specific stats
	//

	if ( a->argc == 4 )
	{
		// show stats for a particular conference
		conference_show_stats_name( a->fd, a->argv[3] ) ;
	}

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_list(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference list";
		e->usage = "usage: conference list {<conference_name>}\n"
			" list members of a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	int index;

	if ( a->argc < 2 )
		return CLI_SHOWUSAGE ;

	if (a->argc >= 3)
	{
		for (index = 2; index < a->argc; index++)
		{
			// get the conference name
			const char* name = a->argv[index] ;
			show_conference_list( a->fd, name );
		}
	}
	else
	{
		show_conference_stats(a->fd);
	}
	return CLI_SUCCESS ;
}


static char *handle_cli_app_conference_kick(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference kick";
		e->usage = "usage: conference kick <conference> <member id>\n"
			" kick member <member id> from conference <conference>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}
	
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE ;

	// get the conference name
	const char* name = a->argv[2] ;

	int member_id;
	sscanf(a->argv[3], "%d", &member_id);

	int res = kick_member( name, member_id );

	if (res) ast_cli( a->fd, "User #: %d kicked\n", member_id) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_kickchannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference kickchannel";
		e->usage = "usage: conference kickchannel <conference_name> <channel>\n"
			" kick channel from conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}
	
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE ;

	const char *name = a->argv[2] ;
	const char *channel = a->argv[3];

	int res = kick_channel( name, channel );

	if ( !res )
	{
		ast_cli( a->fd, "Cannot kick channel %s in conference %s\n", channel, name);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_exit(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference exit";
		e->usage = "usage: conference exit <channel>\n"
			" exit channel from any conference where it in\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 3 )
		return CLI_SHOWUSAGE ;

	const char *channel = a->argv[2];

	struct ast_conf_member *member = find_member(channel, 1);
	if(!member)
	{
		ast_cli( a->fd, "Member %s not found\n", channel);
		return CLI_FAILURE;
	}
	const char * name = member->conf_name;
	int res = kick_channel( name, channel );
	
	if ( !res )
	{
		ast_cli( a->fd, "Cannot exit channel %s from conference %s\n", channel, name);
		ast_mutex_unlock(&member->lock);
		return CLI_FAILURE;
	}

	ast_mutex_unlock( &member->lock ) ;
	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_mute(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference mute";
		e->usage = "usage: conference mute <conference_name> <member id>\n"
			" mute member in a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 4 )
		return CLI_SHOWUSAGE ;

	// get the conference name
	const char* name = a->argv[2] ;

	int member_id;
	sscanf(a->argv[3], "%d", &member_id);

	int res = mute_member( name, member_id );

	if (res) ast_cli( a->fd, "User #: %d muted\n", member_id) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_mutechannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference mutechannel";
		e->usage = "usage: conference mutechannel <channel>\n"
			" mute channel in a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

  	struct ast_conf_member *member;
	char *channel;

	if ( a->argc < 3 )
		return CLI_SHOWUSAGE ;

	channel = a->argv[2];

	member = find_member(channel, 1);
	if(!member) {
	    ast_cli( a->fd, "Member %s not found\n", channel);
	    return CLI_FAILURE;
	}

	member->mute_audio = 1;
	ast_mutex_unlock( &member->lock ) ;

	ast_cli( a->fd, "Channel #: %s muted\n", a->argv[2]) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_viewstream(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference viewstream";
		e->usage = "usage: conference viewstream <conference_name> <member id> <stream no>\n"
			" member <member id> will receive video stream <stream no>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	int res;

	if ( a->argc < 5 )
		return CLI_SHOWUSAGE ;

	// get the conference name
	const char* switch_name = a->argv[2] ;

	int member_id, viewstream_id;
	sscanf(a->argv[3], "%d", &member_id);
	sscanf(a->argv[4], "%d", &viewstream_id);

	res = viewstream_switch( switch_name, member_id, viewstream_id );

	if (res) ast_cli( a->fd, "User #: %d viewing %d\n", member_id, viewstream_id) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_viewchannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference viewchannel";
		e->usage = "usage: conference viewchannel <conference_name> <dest channel> <src channel>\n"
			" channel <dest channel> will receive video stream <src channel>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	int res;

	if ( a->argc < 5 )
		return CLI_SHOWUSAGE ;

	// get the conference name
	const char* switch_name = a->argv[2] ;

	res = viewchannel_switch( switch_name, a->argv[3], a->argv[4] );

	if (res) ast_cli( a->fd, "Channel #: %s viewing %s\n", a->argv[3], a->argv[4]) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_unmute(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference unmute";
		e->usage = "usage: conference unmute <conference_name> <member id>\n"
			" unmute member in a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	if ( a->argc < 4 )
		return CLI_SHOWUSAGE ;

	// get the conference name
	const char* name = a->argv[2] ;

	int member_id;
	sscanf(a->argv[3], "%d", &member_id);

	int res = unmute_member( name, member_id );

	if (res) ast_cli( a->fd, "User #: %d unmuted\n", member_id) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_unmutechannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference unmutechannel";
		e->usage = "usage: conference unmutechannel <channel>\n"
			" unmute channel in a conference\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	struct ast_conf_member *member;
	char *channel;

	if ( a->argc < 3 )
		return CLI_SHOWUSAGE ;

	channel = a->argv[2];

	member = find_member(channel, 1);
	if(!member) {
	    ast_cli( a->fd, "Member %s not found\n", channel);
	    return CLI_FAILURE;
	}

	member->mute_audio = 0;
	ast_mutex_unlock( &member->lock ) ;

	ast_cli( a->fd, "Channel #: %s unmuted\n", a->argv[2]) ;

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_play_sound(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference play sound";
		e->usage = "usage: conference play sound <channel-id> <sound-file> [mute]\n"
			" play sound <sound-file> to conference member <channel-id>.\n"
			" If mute is specified, all other audio is muted while the sound is played back.\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	char *channel, *file;
	int mute = 0;

	if ( a->argc < 5 )
		return CLI_SHOWUSAGE ;

	channel = a->argv[3];
	file = a->argv[4];

	if(a->argc > 5 && !strcmp(a->argv[5], "mute"))
	    mute = 1;

	int res = play_sound_channel(a->fd, channel, file, mute);

	if ( !res )
	{
		ast_cli( a->fd, "Sound playback failed failed\n");
		return CLI_FAILURE;
	}
	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_stop_sounds(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference play sound";
		e->usage = "usage: conference stop sounds <channel-id>\n"
			" stop sounds for conference member <channel-id>.\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	char *channel;

	if ( a->argc < 4 )
		return CLI_SHOWUSAGE ;

	channel = a->argv[3];

	int res = stop_sound_channel( a->fd, channel);

	if ( !res )
	{
		ast_cli( a->fd, "Sound stop failed failed\n");
		return CLI_FAILURE;
	}
	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_end(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference end";
		e->usage = "usage: conference end <conference name>\n"
			" ends a conference.\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check the args length
	if ( a->argc < 3 )
		return CLI_SHOWUSAGE ;

	// conference name
	const char* name = a->argv[2] ;

	// get the conference
	if ( end_conference( name, 1 ) != 0 )
	{
		ast_cli( a->fd, "unable to end the conference, name => %s\n", name ) ;
		return CLI_SHOWUSAGE ;
	}

	return CLI_SUCCESS ;
}

static char *handle_cli_app_conference_lock(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
	e->command = "conference lock";
	e->usage = "usage: conference lock <conference name> <member id>\n"
		" locks incoming video stream for conference <conference name> to member <member id>\n";
	return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	int member;
	sscanf(a->argv[3], "%d", &member);

	int res = lock_conference(conference, member);

	if ( !res )
	{
		ast_cli( a->fd, "Locking failed\n");
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_lockchannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference lockchannel";
		e->usage = "usage: conference lockchannel <conference name> <channel>\n"
			" locks incoming video stream for conference <conference name> to channel <channel>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	const char *channel = a->argv[3];

	int res = lock_conference_channel(conference, channel);

	if ( !res )
	{
		ast_cli( a->fd, "Locking failed\n");
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_unlock(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference unlock";
		e->usage = "usage: conference unlock <conference name>\n"
			" unlocks conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 3 )
		return CLI_SHOWUSAGE;


	const char *conference = a->argv[2];

	int res = unlock_conference(conference);

	if ( !res )
	{
		ast_cli( a->fd, "Unlocking failed\n");
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_set_default(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference set default";
		e->usage = "usage: conference set default <conference name> <member id>\n"
			" sets the default video source for conference <conference name> to member <member id>\n"
			" Use a negative value for member if you want to clear the default\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	int member;
	sscanf(a->argv[4], "%d", &member);

	int res = set_default_id(conference, member);

	if ( !res )
	{
		ast_cli( a->fd, "Setting default video id failed\n");
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_set_defaultchannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference set defaultchannel";
		e->usage = "usage: conference set defaultchannel <conference name> <channel>\n"
			" sets the default video source channel for conference <conference name> to channel <channel>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	const char *channel = a->argv[4];

	int res = set_default_channel(conference, channel);

	if ( !res )
	{
		ast_cli( a->fd, "Setting default video id failed\n");
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_video_mute(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference video mute";
		e->usage = "usage: conference video mute <conference name> <member id>\n"
			" mutes video from member <member id> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	int member;
	sscanf(a->argv[4], "%d", &member);

	int res = video_mute_member(conference, member);

	if ( !res )
	{
		ast_cli( a->fd, "Muting video from member %d failed\n", member);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_video_unmute(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference video unmute";
		e->usage = "usage: conference video unmute <conference name> <member id>\n"
			" unmutes video from member <member id> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	int member;
	sscanf(a->argv[4], "%d", &member);

	int res = video_unmute_member(conference, member);

	if ( !res )
	{
		ast_cli( a->fd, "Unmuting video from member %d failed\n", member);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_video_mutechannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference video mutechannel";
		e->usage = "usage: conference video mutechannel <conference name> <channel>\n"
			" mutes video from channel <channel> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	const char *channel = a->argv[4];

	int res = video_mute_channel(conference, channel);

	if ( !res )
	{
		ast_cli( a->fd, "Muting video from channel %s failed\n", channel);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_video_unmutechannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference video unmutechannel";
		e->usage = "usage: conference video unmutechannel <conference name> <channel>\n"
			" unmutes video from channel <channel> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[3];
	const char *channel = a->argv[4];

	int res = video_unmute_channel(conference, channel);

	if ( !res )
	{
		ast_cli( a->fd, "Unmuting video from channel %s failed\n", channel);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_text(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference text";
		e->usage = "usage: conference text <conference name> <member id> <text>\n"
			" Sends text message <text> to member <member id> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	int member;
	sscanf(a->argv[3], "%d", &member);
	const char *text = a->argv[4];

	int res = send_text(conference, member, text);

	if ( !res )
	{
		ast_cli( a->fd, "Sending a text message to member %d failed\n", member);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_textchannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference textchannel";
		e->usage = "usage: conference textchannel <conference name> <channel> <text>\n"
			" Sends text message <text> to channel <channel> in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 5 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	const char *channel = a->argv[3];
	const char *text = a->argv[4];

	int res = send_text_channel(conference, channel, text);

	if ( !res )
	{
		ast_cli( a->fd, "Sending a text message to channel %s failed\n", channel);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_textbroadcast(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference textbroadcast";
		e->usage = "usage: conference textbroadcast <conference name> <text>\n"
			" Sends text message <text> to all members in conference <conference name>\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	const char *text = a->argv[3];

	int res = send_text_broadcast(conference, text);

	if ( !res )
	{
		ast_cli( a->fd, "Sending a text broadcast to conference %s failed\n", conference);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_drive(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference drive";
		e->usage = "usage: conference drive <conference name> <source member> [destination member]\n"
			" Drives VAD video switching of <destination member> using audio from <source member> in conference <conference name>\n"
			" If destination is missing or negative, break existing association\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	int src_member = -1;
	int dst_member = -1;
	sscanf(a->argv[3], "%d", &src_member);
	if ( a->argc > 4 )
		sscanf(a->argv[4], "%d", &dst_member);

	int res = drive(conference, src_member, dst_member);

	if ( !res )
	{
		ast_cli( a->fd, "Pairing members %d and %d failed\n", src_member, dst_member);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static char *handle_cli_app_conference_drivechannel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch ( cmd ) {
	case CLI_INIT:
		e->command = "conference drivechannel";
		e->usage = "usage: conference drive <conference name> <source channel> [destination channel]\n"
			" Drives VAD video switching of <destination member> using audio from <source channel> in conference <conference channel>\n"
			" If destination is missing, break existing association\n";
		return NULL;
	case CLI_GENERATE:
		return NULL;
	}

	// check args
	if ( a->argc < 4 )
		return CLI_SHOWUSAGE;

	const char *conference = a->argv[2];
	const char *src_channel = a->argv[3];
	const char *dst_channel = NULL;
	if ( a->argc > 4 )
		dst_channel = a->argv[4];

	int res = drive_channel(conference, src_channel, dst_channel);

	if ( !res )
	{
		ast_cli( a->fd, "Pairing channels %s and %s failed\n", src_channel, dst_channel);
		return CLI_FAILURE;
	}

	return CLI_SUCCESS;
}

static struct ast_cli_entry app_conference_clis[] = {
	AST_CLI_DEFINE(handle_cli_app_conference_restart, "Restart a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_debug, "Enable debugging for a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_show_stats, "Display stats for active conferences"),
	AST_CLI_DEFINE(handle_cli_app_conference_list, "List members of a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_kick, "Kick member from a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_kickchannel, "Kick channel from conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_exit, "Exit channel from any conference where it in"),
	AST_CLI_DEFINE(handle_cli_app_conference_mute, "Mute member in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_mutechannel, "Mute channel in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_viewstream, "Switch view in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_viewchannel, "Switch channel in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_unmute, "Unmute member in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_unmutechannel, "Unmute channel in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_play_sound, "Play a sound to a conference member"),
	AST_CLI_DEFINE(handle_cli_app_conference_stop_sounds, "Stop sounds for a conference member"),
	AST_CLI_DEFINE(handle_cli_app_conference_end, "Stops a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_lock, "Locks incoming video to a member"),
	AST_CLI_DEFINE(handle_cli_app_conference_lockchannel, "Locks incoming video to a channel"),
	AST_CLI_DEFINE(handle_cli_app_conference_unlock, "Unlocks conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_set_default, "Sets default video sourcee"),
	AST_CLI_DEFINE(handle_cli_app_conference_set_defaultchannel, "Sets default video source channel"),
	AST_CLI_DEFINE(handle_cli_app_conference_video_mute, "Mutes video from a member"),
	AST_CLI_DEFINE(handle_cli_app_conference_video_unmute, "Unmutes video from a member"),
	AST_CLI_DEFINE(handle_cli_app_conference_video_mutechannel, "Mutes video from a channel"),
	AST_CLI_DEFINE(handle_cli_app_conference_video_unmutechannel, "Unmutes video from a channel"),
	AST_CLI_DEFINE(handle_cli_app_conference_text, "Sends a text message to a member"),
	AST_CLI_DEFINE(handle_cli_app_conference_textchannel, "Sends a text message to a channel"),
	AST_CLI_DEFINE(handle_cli_app_conference_textbroadcast, "Sends a text message to all members in a conference"),
	AST_CLI_DEFINE(handle_cli_app_conference_drive, "Pairs two members to drive VAD-based video switching"),
	AST_CLI_DEFINE(handle_cli_app_conference_drivechannel, "Pairs two channels to drive VAD-based video switching"),
};

//
// E.BUU - Manager conference end. Additional option to just kick everybody out
// without hangin up channels
//
int manager_conference_end(struct mansession *s, const struct message *m)
{
	const char *confname = astman_get_header(m,"Conference");
	int hangup = 1;

	const char * h =  astman_get_header(m, "Hangup");
	if (h)
	{
		hangup = atoi(h);
	}

	ast_log( LOG_NOTICE, "Terminating conference %s on manager's request. Hangup: %s.\n", confname, hangup?"YES":"NO" );
        if ( end_conference( confname, hangup ) != 0 )
        {
		ast_log( LOG_ERROR, "manager end conf: unable to terminate conference %s.\n", confname );
		astman_send_error(s, m, "Failed to terminate\r\n");
		return RESULT_FAILURE;
	}

	astman_send_ack(s, m, "Conference terminated");
	return RESULT_SUCCESS;
}

//
// cli initialization function
//

void register_conference_cli( void )
{
	ast_cli_register_multiple(app_conference_clis, sizeof(app_conference_clis) / sizeof(struct ast_cli_entry));
	ast_manager_register( "ConferenceList", 0, manager_conference_list, "Conference List" );
	ast_manager_register( "ConferenceEnd", EVENT_FLAG_CALL, manager_conference_end, "Terminate a conference" );

}

void unregister_conference_cli( void )
{
	ast_cli_unregister_multiple(app_conference_clis, sizeof(app_conference_clis) / sizeof(struct ast_cli_entry));
	ast_manager_unregister( "ConferenceList" );
	ast_manager_unregister( "ConferenceEnd" );
}
